package com.zulutrade.sample.binaryoptions.trading.client.trading;

import java.util.List;

import com.zulutrade.sample.binaryoptions.trading.client.configuration.ZuluTradeAuthenticationInfo;
import com.zulutrade.sample.binaryoptions.trading.client.configuration.ZuluTradingServerConfig;
import com.zulutrade.sample.binaryoptions.trading.client.json.ZuluJsonSerializer;
import com.zulutrade.sample.binaryoptions.trading.client.restclient.RestClient;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.Position;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.TraderStatistics;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.RegularOption;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.request.OpenRegularPositionRequest;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.request.OpenSixtySecondsPositionRequest;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.request.RegularOptionsRequest;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.response.AssetsResponse;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.response.BalanceResponse;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.response.PositionResponse;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.response.PositionsResponse;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.response.TraderStatisticsResponse;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.response.RegularOptionsResponse;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.response.TradingResponse;
import com.zulutrade.sample.binaryoptions.trading.client.trading.dto.rest.response.ZuluPerformanceResponse;

/**
 * BinaryOptionsRestTradingService, the rest implementation of the BinaryOptionsTradingService
 *
 * @author ZuluTrade
 * @version 1.0
 * @since 2015-06-15
*/
public class BinaryOptionsRestTradingService implements BinaryOptionsTradingService {

	ZuluTradingServerConfig config; 
	ZuluTradeAuthenticationInfo authenticationsInfo;
		
	public BinaryOptionsRestTradingService(ZuluTradingServerConfig config, ZuluTradeAuthenticationInfo authenticationsInfo) {
		this.config = config;
		this.authenticationsInfo = authenticationsInfo;
	}

	@Override
	public List<String> requestAssets() throws TradingClientException {		
		String responseStr = RestClient.get(config, authenticationsInfo, TradingRequestMethod.REQUEST_ASSETS); 
		AssetsResponse tradingResponse = getTradingResponseFromString(responseStr, AssetsResponse.class);
		return getResultFromResponse(tradingResponse);
	}

	@Override
	public double requestBalance() throws TradingClientException {
		String responseStr = RestClient.get(config, authenticationsInfo, TradingRequestMethod.REQUEST_BALANCE); 
		BalanceResponse tradingResponse = getTradingResponseFromString(responseStr, BalanceResponse.class);
		return getResultFromResponse(tradingResponse);
	}

	@Override
	public List<Position> requestOpenPositions() throws TradingClientException {
		String responseStr = RestClient.get(config, authenticationsInfo, TradingRequestMethod.REQUEST_OPEN_POSITIONS); 
		PositionsResponse tradingResponse = getTradingResponseFromString(responseStr, PositionsResponse.class);
		return getResultFromResponse(tradingResponse);
	}

	@Override
	public List<Position> requestClosedPositions() throws TradingClientException {
		String responseStr = RestClient.get(config, authenticationsInfo, TradingRequestMethod.REQUEST_CLOSED_POSITIONS); 
		PositionsResponse tradingResponse = getTradingResponseFromString(responseStr, PositionsResponse.class);
		return getResultFromResponse(tradingResponse);
	}
	
	@Override
	public List<Position> requestClosedPositionsByDate(String startDate, String endDate) throws TradingClientException {
		String responseStr = RestClient.get(config, authenticationsInfo, TradingRequestMethod.REQUEST_CLOSED_POSITIONS+"/?startDate="+startDate+"&endDate="+endDate); 
		PositionsResponse tradingResponse = getTradingResponseFromString(responseStr, PositionsResponse.class);
		return getResultFromResponse(tradingResponse);
	}

	@Override
	public TraderStatistics requestTraderStatistics() throws TradingClientException {
		String responseStr = RestClient.get(config, authenticationsInfo, TradingRequestMethod.REQUEST_TRADER_STATISTICS); 
		TraderStatisticsResponse tradingResponse = getTradingResponseFromString(responseStr, TraderStatisticsResponse.class);
		return getResultFromResponse(tradingResponse);
	}

	@Override
	public List<TraderStatistics> requestTopTrades(int number) throws TradingClientException {
		String responseStr = RestClient.get(config, authenticationsInfo, TradingRequestMethod.REQUEST_ZULU_PERFORMANCE+"/"+number);
		ZuluPerformanceResponse tradingResponse = getTradingResponseFromString(responseStr, ZuluPerformanceResponse.class);
		return getResultFromResponse(tradingResponse);
	}
	
	@Override
	public Position openSixtySecondsPosition(OpenSixtySecondsPositionRequest openRequest) throws TradingClientException {
		String responseStr = RestClient.post(config, authenticationsInfo, TradingRequestMethod.OPEN_SIXTY_SECONDS_POSITION, openRequest);
		PositionResponse tradingResponse = getTradingResponseFromString(responseStr, PositionResponse.class); 
		return getResultFromResponse(tradingResponse);
	}

	@Override
	public Position openRegularPosition(OpenRegularPositionRequest openRequest) throws TradingClientException {
		String responseStr = RestClient.post(config, authenticationsInfo, TradingRequestMethod.OPEN_REGULAR_POSITION, openRequest);
		PositionResponse tradingResponse = getTradingResponseFromString(responseStr, PositionResponse.class); 
		return getResultFromResponse(tradingResponse);
	}

	@Override
	public List<RegularOption> requestRegularOptions(RegularOptionsRequest request) throws TradingClientException {
		String responseStr = RestClient.post(config, authenticationsInfo, TradingRequestMethod.REQUEST_REGULAR_OPTIONS, request);
		RegularOptionsResponse tradingResponse = getTradingResponseFromString(responseStr, RegularOptionsResponse.class); 
		return getResultFromResponse(tradingResponse);
	}
	
	<T extends TradingResponse> T getTradingResponseFromString(String responseStr, Class<T> responseClass) throws TradingClientException{
		T response;
		try {
			response = ZuluJsonSerializer.jsonDeserialize(responseStr, responseClass);
		} catch (Exception ex) {
			throw new TradingClientException(RestClientError.RESPONSE_DESERIALIZATION_ERROR);
		}
		return response; 
	}
	
	@SuppressWarnings("unchecked")
	<T> T getResultFromResponse(TradingResponse response) throws TradingClientException{
		if(!response.isSuccess()){
			throw new TradingClientException(response.getError()); 
		}
		return (T) response.getResult(); 
	}

}
